<?php
/*
Plugin Name: Stereopix Media Viewer
Description: Display WordPress Media Library images in the Stereopix embedded viewer with stereoscopic field support.
Version: 1.9
Author: Your Name
License: GPL-2.0+
*/

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

// Add CORS headers for Media Library images
add_filter('wp_headers', 'stereopix_add_cors_headers');
function stereopix_add_cors_headers($headers) {
    if (isset($_SERVER['HTTP_ORIGIN']) && $_SERVER['HTTP_ORIGIN'] === 'https://stereopix.net') {
        $headers['Access-Control-Allow-Origin'] = 'https://stereopix.net';
        $headers['Access-Control-Allow-Methods'] = 'GET';
    }
    return $headers;
}

// Add stereoscopic field to attachment edit screen
add_filter('attachment_fields_to_edit', 'stereopix_add_stereoscopic_field', 10, 2);
function stereopix_add_stereoscopic_field($form_fields, $post) {
    $is_stereoscopic = get_post_meta($post->ID, '_is_stereoscopic', true);
    $form_fields['is_stereoscopic'] = [
        'label' => __('Is Stereoscopic?', 'stereopix-media-viewer'),
        'input' => 'html',
        'html'  => '<input type="checkbox" name="attachments[' . $post->ID . '][is_stereoscopic]" id="attachments-' . $post->ID . '-is_stereoscopic" ' . checked($is_stereoscopic, '1', false) . ' value="1" />',
        'value' => $is_stereoscopic,
        'helps' => __('Check if this image is stereoscopic (e.g., parallel, cross, anaglyph).', 'stereopix-media-viewer'),
    ];
    return $form_fields;
}

// Save stereoscopic field
add_filter('attachment_fields_to_save', 'stereopix_save_stereoscopic_field', 10, 2);
function stereopix_save_stereoscopic_field($post, $attachment) {
    if (isset($attachment['is_stereoscopic'])) {
        update_post_meta($post['ID'], '_is_stereoscopic', '1');
    } else {
        delete_post_meta($post['ID'], '_is_stereoscopic');
    }
    return $post;
}

// Enqueue scripts and styles for Gutenberg block
add_action('enqueue_block_editor_assets', 'stereopix_enqueue_block_assets');
function stereopix_enqueue_block_assets() {
    wp_enqueue_script(
        'stereopix-block',
        plugins_url('block.js', __FILE__),
        ['wp-blocks', 'wp-element', 'wp-block-editor', 'wp-components', 'wp-i18n'],
        filemtime(plugin_dir_path(__FILE__) . 'block.js')
    );

    wp_enqueue_style(
        'stereopix-block-editor',
        plugins_url('block.css', __FILE__),
        ['wp-edit-blocks'],
        filemtime(plugin_dir_path(__FILE__) . 'block.css')
    );
}

// Register block
add_action('init', 'stereopix_register_block');
function stereopix_register_block() {
    register_block_type('stereopix/media-viewer', [
        'editor_script' => 'stereopix-block',
        'render_callback' => 'stereopix_render_block',
        'attributes' => [
            'mediaItems' => [
                'type' => 'array',
                'default' => [],
                'items' => [
                    'type' => 'object',
                    'properties' => [
                        'id' => ['type' => 'number'],
                        'url' => ['type' => 'string'],
                        'title' => ['type' => 'string'],
                        'caption' => ['type' => 'string'],
                    ],
                ],
            ],
            'format' => ['type' => 'string', 'default' => 'parallel'],
            'width' => ['type' => 'string', 'default' => '100%'],
            'height' => ['type' => 'string', 'default' => '600px'],
            'filterStereoscopic' => ['type' => 'boolean', 'default' => false],
            'initialMediaIndex' => ['type' => 'number', 'default' => 0],
            'showTitle' => ['type' => 'boolean', 'default' => false],
            'showCaption' => ['type' => 'boolean', 'default' => false],
        ],
    ]);
}

// Render block on frontend
function stereopix_render_block($attributes) {
    if (empty($attributes['mediaItems']) || !is_array($attributes['mediaItems'])) {
        return '<p>' . __('Please select at least one image from the Media Library.', 'stereopix-media-viewer') . '</p>';
    }

    // Validate stereoscopic status if filter is enabled
    if ($attributes['filterStereoscopic']) {
        foreach ($attributes['mediaItems'] as $media) {
            $is_stereoscopic = get_post_meta($media['id'], '_is_stereoscopic', true);
            $is_stereoscopic = get_post_meta($media['id'], '_is_stereoscopic', true);
            if (!$is_stereoscopic) {
                return '<p>' . __('One or more selected images are not marked as stereoscopic.', 'stereopix-media-viewer') . '</p>';
            }
        }
    }

    // List of valid Stereopix formats
    $valid_formats = [
        'single_left', 'single_right', 'parallel', 'cross', 'mirrored_left', 'mirrored_right', 'mirrored_both',
        'top_bottom', 'bottom_top', 'top_bottom_anamorphic', 'bottom_top_anamorphic',
        'interleaved_lines_1', 'interleaved_lines_2', 'interleaved_columns_1', 'interleaved_columns_2',
        'checkerboard_1', 'checkerboard_2', 'anaglyph_red_cyan_gray', 'anaglyph_red_cyan_dubois',
        'anaglyph_green_magenta_dubois', 'anaglyph_yellow_blue_dubois', 'sview', 'webxr'
    ];
    $format = in_array($attributes['format'], $valid_formats) ? $attributes['format'] : 'parallel';
    $width = esc_attr($attributes['width']);
    $height = esc_attr($attributes['height']);
    $initial_index = max(0, (int)$attributes['initialMediaIndex']);
    $show_title = (bool)$attributes['showTitle'];
    $show_caption = (bool)$attributes['showCaption'];
    $json_id = 'stereopix_json_' . uniqid();
    $iframe_id = 'stereopix_iframe_' . uniqid();
    $caption_container_id = 'stereopix_caption_' . uniqid();

    // Generate JSON data
    $json_data = [
        'media' => array_map(function($media) use ($format, $show_title, $show_caption) {
            $item = [
                'url' => esc_url($media['url']),
                'preset' => $format
            ];
            if ($show_title || $show_caption) {
                $item['userdata'] = [];
                if ($show_title) {
                    $item['userdata']['title'] = isset($media['title']) ? esc_html($media['title']) : '';
                }
                if ($show_caption) {
                    $item['userdata']['caption'] = isset($media['caption']) ? esc_html($media['caption']) : '';
                }
            }
            return $item;
        }, $attributes['mediaItems']),
        'meta' => [
            'initial' => $initial_index
        ]
    ];

    ob_start();
    ?>
    <div class="stereopix-viewer-container" style="width: <?php echo $width; ?>; margin: 8px 0;">
        <script type="application/json" id="<?php echo esc_attr($json_id); ?>">
            <?php echo wp_json_encode($json_data); ?>
        </script>
        <script>
            window.addEventListener('message', function(e) {
                if (e.origin === 'https://stereopix.net' && e.data.type === 'viewerReady') {
                    var iframe = document.getElementById('<?php echo esc_js($iframe_id); ?>');
                    if (e.source === iframe.contentWindow) {
                        try {
                            var json = JSON.parse(document.getElementById('<?php echo esc_js($json_id); ?>').textContent);
                            // Send media list
                            e.source.postMessage({
                                'stereopix_action': 'list_add_json',
                                'media': json
                            }, 'https://stereopix.net');
                            // Explicitly set the format
                            e.source.postMessage({
                                'stereopix_action': 'set_format',
                                'format': '<?php echo esc_js($format); ?>'
                            }, 'https://stereopix.net');
                            // Periodically check for current media index
                            setInterval(function() {
                                e.source.postMessage({
                                    'stereopix_action': 'get_current_media_index'
                                }, 'https://stereopix.net');
                            }, 1000);
                        } catch (err) {
                            console.error('Stereopix JSON Error: ' + err.message);
                        }
                    }
                }
                // Handle current media index response
                if (e.origin === 'https://stereopix.net' && e.data.type === 'currentMediaIndex') {
                    var container = document.getElementById('<?php echo esc_js($caption_container_id); ?>');
                    if (container && json && json.media && json.media[e.data.index]) {
                        var media = json.media[e.data.index];
                        var title = <?php echo $show_title ? 'media.userdata && media.userdata.title ? media.userdata.title : ""' : '""'; ?>;
                        var caption = <?php echo $show_caption ? 'media.userdata && media.userdata.caption ? media.userdata.caption : ""' : '""'; ?>;
                        var html = '';
                        if (title) {
                            html += '<h3 class="stereopix-title">' + title.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"') + '</h3>';
                        }
                        if (caption) {
                            html += '<p class="stereopix-caption">' + caption.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"') + '</p>';
                        }
                        container.innerHTML = html;
                    }
                }
            }, false);
        </script>
        <iframe
            id="<?php echo esc_attr($iframe_id); ?>"
            title="<?php echo esc_attr__('Stereopix Viewer', 'stereopix-media-viewer'); ?>"
            style="width: 100%; height: <?php echo $height; ?>; border: 2px solid black;"
            allowfullscreen="yes"
            allow="fullscreen; xr-spatial-tracking; accelerometer; gyroscope"
            src="https://stereopix.net/viewer:embed/"></iframe>
        <div id="<?php echo esc_attr($caption_container_id); ?>" class="stereopix-caption-container"></div>
    </div>
    <?php
    return ob_get_clean();
}

// Shortcode for classic editor
add_shortcode('stereopix_viewer', 'stereopix_shortcode');
function stereopix_shortcode($atts) {
    $atts = shortcode_atts([
        'media_ids' => '',
        'format' => 'parallel',
        'width' => '100%',
        'height' => '600px',
        'filter_stereoscopic' => false,
        'initial_index' => 0,
        'show_title' => false,
        'show_caption' => false,
    ], $atts, 'stereopix_viewer');

    if (empty($atts['media_ids'])) {
        return '<p>' . __('Please specify at least one media ID.', 'stereopix-media-viewer') . '</p>';
    }

    // Parse comma-separated media IDs
    $media_ids = array_filter(array_map('intval', explode(',', $atts['media_ids'])));
    if (empty($media_ids)) {
        return '<p>' . __('Invalid media IDs.', 'stereopix-media-viewer') . '</p>';
    }

    // Build media items array
    $media_items = [];
    foreach ($media_ids as $id) {
        $url = wp_get_attachment_url($id);
        if ($url) {
            $attachment = get_post($id);
            $media_items[] = [
                'id' => $id,
                'url' => $url,
                'title' => $attachment ? esc_html($attachment->post_title) : '',
                'caption' => $attachment ? esc_html($attachment->post_excerpt) : '',
            ];
        }
    }

    if (empty($media_items)) {
        return '<p>' . __('No valid media found for the specified IDs.', 'stereopix-media-viewer') . '</p>';
    }

    // List of valid Stereopix formats
    $valid_formats = [
        'single_left', 'single_right', 'parallel', 'cross', 'mirrored_left', 'mirrored_right', 'mirrored_both',
        'top_bottom', 'bottom_top', 'top_bottom_anamorphic', 'bottom_top_anamorphic',
        'interleaved_lines_1', 'interleaved_lines_2', 'interleaved_columns_1', 'interleaved_columns_2',
        'checkerboard_1', 'checkerboard_2', 'anaglyph_red_cyan_gray', 'anaglyph_red_cyan_dubois',
        'anaglyph_green_magenta_dubois', 'anaglyph_yellow_blue_dubois', 'sview', 'webxr'
    ];
    $format = in_array($atts['format'], $valid_formats) ? $atts['format'] : 'parallel';

    return stereopix_render_block([
        'mediaItems' => $media_items,
        'format' => $format,
        'width' => $atts['width'],
        'height' => $atts['height'],
        'filterStereoscopic' => filter_var($atts['filter_stereoscopic'], FILTER_VALIDATE_BOOLEAN),
        'initialMediaIndex' => max(0, (int)$atts['initial_index']),
        'showTitle' => filter_var($atts['show_title'], FILTER_VALIDATE_BOOLEAN),
        'showCaption' => filter_var($atts['show_caption'], FILTER_VALIDATE_BOOLEAN),
    ]);
}

// Add settings page
add_action('admin_menu', 'stereopix_add_settings_page');
function stereopix_add_settings_page() {
    add_options_page(
        __('Stereopix Viewer Settings', 'stereopix-media-viewer'),
        __('Stereopix Viewer', 'stereopix-media-viewer'),
        'manage_options',
        'stereopix-settings',
        'stereopix_render_settings_page'
    );
}

// Render settings page
function stereopix_render_settings_page() {
    ?>
    <div class="wrap">
        <h1><?php _e('Stereopix Viewer Settings', 'stereopix-media-viewer'); ?></h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('stereopix_settings');
            do_settings_sections('stereopix-settings');
            submit_button();
            ?>
        </form>
        <p><?php _e('Note: Ensure your server allows CORS requests from https://stereopix.net for Media Library images.', 'stereopix-media-viewer'); ?></p>
    </div>
    <?php
}

// Register settings
add_action('admin_init', 'stereopix_register_settings');
function stereopix_register_settings() {
    register_setting('stereopix_settings', 'stereopix_default_format', ['default' => 'parallel']);
    add_settings_section('stereopix_main', __('Default Settings', 'stereopix-media-viewer'), null, 'stereopix-settings');
    add_settings_field(
        'stereopix_default_format',
        __('Default Format', 'stereopix-media-viewer'),
        'stereopix_default_format_callback',
        'stereopix-settings',
        'stereopix_main'
    );
}

// Settings field callback
function stereopix_default_format_callback() {
    $format = get_option('stereopix_default_format', 'parallel');
    ?>
    <select name="stereopix_default_format">
        <option value="single_left" <?php selected($format, 'single_left'); ?>><?php _e('Single Left', 'stereopix-media-viewer'); ?></option>
        <option value="single_right" <?php selected($format, 'single_right'); ?>><?php _e('Single Right', 'stereopix-media-viewer'); ?></option>
        <option value="parallel" <?php selected($format, 'parallel'); ?>><?php _e('Parallel', 'stereopix-media-viewer'); ?></option>
        <option value="cross" <?php selected($format, 'cross'); ?>><?php _e('Cross', 'stereopix-media-viewer'); ?></option>
        <option value="mirrored_left" <?php selected($format, 'mirrored_left'); ?>><?php _e('Mirrored Left', 'stereopix-media-viewer'); ?></option>
        <option value="mirrored_right" <?php selected($format, 'mirrored_right'); ?>><?php _e('Mirrored Right', 'stereopix-media-viewer'); ?></option>
        <option value="mirrored_both" <?php selected($format, 'mirrored_both'); ?>><?php _e('Mirrored Both', 'stereopix-media-viewer'); ?></option>
        <option value="top_bottom" <?php selected($format, 'top_bottom'); ?>><?php _e('Top-Bottom', 'stereopix-media-viewer'); ?></option>
        <option value="bottom_top" <?php selected($format, 'bottom_top'); ?>><?php _e('Bottom-Top', 'stereopix-media-viewer'); ?></option>
        <option value="top_bottom_anamorphic" <?php selected($format, 'top_bottom_anamorphic'); ?>><?php _e('Top-Bottom Anamorphic', 'stereopix-media-viewer'); ?></option>
        <option value="bottom_top_anamorphic" <?php selected($format, 'bottom_top_anamorphic'); ?>><?php _e('Bottom-Top Anamorphic', 'stereopix-media-viewer'); ?></option>
        <option value="interleaved_lines_1" <?php selected($format, 'interleaved_lines_1'); ?>><?php _e('Interleaved Lines 1', 'stereopix-media-viewer'); ?></option>
        <option value="interleaved_lines_2" <?php selected($format, 'interleaved_lines_2'); ?>><?php _e('Interleaved Lines 2', 'stereopix-media-viewer'); ?></option>
        <option value="interleaved_columns_1" <?php selected($format, 'interleaved_columns_1'); ?>><?php _e('Interleaved Columns 1', 'stereopix-media-viewer'); ?></option>
        <option value="interleaved_columns_2" <?php selected($format, 'interleaved_columns_2'); ?>><?php _e('Interleaved Columns 2', 'stereopix-media-viewer'); ?></option>
        <option value="checkerboard_1" <?php selected($format, 'checkerboard_1'); ?>><?php _e('Checkerboard 1', 'stereopix-media-viewer'); ?></option>
        <option value="checkerboard_2" <?php selected($format, 'checkerboard_2'); ?>><?php _e('Checkerboard 2', 'stereopix-media-viewer'); ?></option>
        <option value="anaglyph_red_cyan_gray" <?php selected($format, 'anaglyph_red_cyan_gray'); ?>><?php _e('Anaglyph Red/Cyan Gray', 'stereopix-media-viewer'); ?></option>
        <option value="anaglyph_red_cyan_dubois" <?php selected($format, 'anaglyph_red_cyan_dubois'); ?>><?php _e('Anaglyph Red/Cyan Dubois', 'stereopix-media-viewer'); ?></option>
        <option value="anaglyph_green_magenta_dubois" <?php selected($format, 'anaglyph_green_magenta_dubois'); ?>><?php _e('Anaglyph Green/Magenta Dubois', 'stereopix-media-viewer'); ?></option>
        <option value="anaglyph_yellow_blue_dubois" <?php selected($format, 'anaglyph_yellow_blue_dubois'); ?>><?php _e('Anaglyph Yellow/Blue Dubois', 'stereopix-media-viewer'); ?></option>
        <option value="sview" <?php selected($format, 'sview'); ?>><?php _e('sView', 'stereopix-media-viewer'); ?></option>
        <option value="webxr" <?php selected($format, 'webxr'); ?>><?php _e('WebXR', 'stereopix-media-viewer'); ?></option>
    </select>
    <?php
}